@@ -1,9 +1,16 @@
 #[cfg(all(
     unix,
     not(mio_unsupported_force_poll_poll),
-    not(any(target_os = "solaris", target_os = "vita"))
+    not(any(
+        target_os = "espidf",
+        target_os = "haiku",
+        target_os = "hermit",
+        target_os = "nto",
+        target_os = "solaris",
+        target_os = "vita"
+    )),
 ))]
-use std::os::unix::io::{AsRawFd, RawFd};
+use std::os::fd::{AsRawFd, RawFd};
 #[cfg(all(debug_assertions, not(target_os = "wasi")))]
 use std::sync::atomic::{AtomicBool, Ordering};
 #[cfg(all(debug_assertions, not(target_os = "wasi")))]
@@ -430,7 +437,14 @@ impl Poll {
 #[cfg(all(
     unix,
     not(mio_unsupported_force_poll_poll),
-    not(any(target_os = "solaris", target_os = "vita"))
+    not(any(
+        target_os = "espidf",
+        target_os = "haiku",
+        target_os = "hermit",
+        target_os = "nto",
+        target_os = "solaris",
+        target_os = "vita"
+    )),
 ))]
 impl AsRawFd for Poll {
     fn as_raw_fd(&self) -> RawFd {
@@ -721,7 +735,14 @@ impl fmt::Debug for Registry {
 #[cfg(all(
     unix,
     not(mio_unsupported_force_poll_poll),
-    not(any(target_os = "solaris", target_os = "vita"))
+    not(any(
+        target_os = "espidf",
+        target_os = "haiku",
+        target_os = "hermit",
+        target_os = "nto",
+        target_os = "solaris",
+        target_os = "vita"
+    )),
 ))]
 impl AsRawFd for Registry {
     fn as_raw_fd(&self) -> RawFd {
@@ -733,7 +754,13 @@ cfg_os_poll! {
     #[cfg(all(
         unix,
         not(mio_unsupported_force_poll_poll),
-        not(any(target_os = "solaris", target_os = "vita")),
+        not(any(
+            target_os = "espidf",
+            target_os = "hermit",
+            target_os = "nto",
+            target_os = "solaris",
+            target_os = "vita"
+        )),
     ))]
     #[test]
     pub fn as_raw_fd() {
