@@ -1,10 +1,9 @@
 use crate::{Interest, Token};
-use log::error;
-use std::mem::MaybeUninit;
+use std::mem::{self, MaybeUninit};
 use std::ops::{Deref, DerefMut};
-use std::os::unix::io::{AsRawFd, RawFd};
+use std::os::fd::{AsRawFd, FromRawFd, OwnedFd, RawFd};
 #[cfg(debug_assertions)]
-use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
+use std::sync::atomic::{AtomicUsize, Ordering};
 use std::time::Duration;
 use std::{cmp, io, ptr, slice};
 
@@ -21,7 +20,13 @@ type Count = libc::size_t;
 // Type of the `filter` field in the `kevent` structure.
 #[cfg(any(target_os = "dragonfly", target_os = "freebsd", target_os = "openbsd"))]
 type Filter = libc::c_short;
-#[cfg(any(target_os = "macos", target_os = "ios"))]
+#[cfg(any(
+    target_os = "ios",
+    target_os = "macos",
+    target_os = "tvos",
+    target_os = "visionos",
+    target_os = "watchos"
+))]
 type Filter = i16;
 #[cfg(target_os = "netbsd")]
 type Filter = u32;
@@ -29,22 +34,17 @@ type Filter = u32;
 // Type of the `flags` field in the `kevent` structure.
 #[cfg(any(target_os = "dragonfly", target_os = "freebsd", target_os = "openbsd"))]
 type Flags = libc::c_ushort;
-#[cfg(any(target_os = "macos", target_os = "ios"))]
+#[cfg(any(
+    target_os = "ios",
+    target_os = "macos",
+    target_os = "tvos",
+    target_os = "visionos",
+    target_os = "watchos"
+))]
 type Flags = u16;
 #[cfg(target_os = "netbsd")]
 type Flags = u32;
 
-// Type of the `data` field in the `kevent` structure.
-#[cfg(any(
-    target_os = "dragonfly",
-    target_os = "freebsd",
-    target_os = "ios",
-    target_os = "macos"
-))]
-type Data = libc::intptr_t;
-#[cfg(any(target_os = "netbsd", target_os = "openbsd"))]
-type Data = i64;
-
 // Type of the `udata` field in the `kevent` structure.
 #[cfg(not(target_os = "netbsd"))]
 type UData = *mut libc::c_void;
@@ -57,9 +57,8 @@ macro_rules! kevent {
             ident: $id as libc::uintptr_t,
             filter: $filter as Filter,
             flags: $flags,
-            fflags: 0,
-            data: 0,
             udata: $data as UData,
+            ..unsafe { mem::zeroed() }
         }
     };
 }
@@ -68,38 +67,33 @@ macro_rules! kevent {
 pub struct Selector {
     #[cfg(debug_assertions)]
     id: usize,
-    kq: RawFd,
-    #[cfg(debug_assertions)]
-    has_waker: AtomicBool,
+    kq: OwnedFd,
 }
 
 impl Selector {
     pub fn new() -> io::Result<Selector> {
-        syscall!(kqueue())
-            .and_then(|kq| syscall!(fcntl(kq, libc::F_SETFD, libc::FD_CLOEXEC)).map(|_| kq))
-            .map(|kq| Selector {
-                #[cfg(debug_assertions)]
-                id: NEXT_ID.fetch_add(1, Ordering::Relaxed),
-                kq,
-                #[cfg(debug_assertions)]
-                has_waker: AtomicBool::new(false),
-            })
+        // SAFETY: `kqueue(2)` ensures the fd is valid.
+        let kq = unsafe { OwnedFd::from_raw_fd(syscall!(kqueue())?) };
+        syscall!(fcntl(kq.as_raw_fd(), libc::F_SETFD, libc::FD_CLOEXEC))?;
+        Ok(Selector {
+            #[cfg(debug_assertions)]
+            id: NEXT_ID.fetch_add(1, Ordering::Relaxed),
+            kq,
+        })
     }
 
     pub fn try_clone(&self) -> io::Result<Selector> {
-        syscall!(fcntl(self.kq, libc::F_DUPFD_CLOEXEC)).map(|kq| Selector {
+        self.kq.try_clone().map(|kq| Selector {
             // It's the same selector, so we use the same id.
             #[cfg(debug_assertions)]
             id: self.id,
             kq,
-            #[cfg(debug_assertions)]
-            has_waker: AtomicBool::new(self.has_waker.load(Ordering::Acquire)),
         })
     }
 
     pub fn select(&self, events: &mut Events, timeout: Option<Duration>) -> io::Result<()> {
         let timeout = timeout.map(|to| libc::timespec {
-            tv_sec: cmp::min(to.as_secs(), libc::time_t::max_value() as u64) as libc::time_t,
+            tv_sec: cmp::min(to.as_secs(), libc::time_t::MAX as u64) as libc::time_t,
             // `Duration::subsec_nanos` is guaranteed to be less than one
             // billion (the number of nanoseconds in a second), making the
             // cast to i32 safe. The cast itself is needed for platforms
@@ -113,7 +107,7 @@ impl Selector {
 
         events.clear();
         syscall!(kevent(
-            self.kq,
+            self.kq.as_raw_fd(),
             ptr::null(),
             0,
             events.as_mut_ptr(),
@@ -163,7 +157,7 @@ impl Selector {
             // the array.
             slice::from_raw_parts_mut(changes[0].as_mut_ptr(), n_changes)
         };
-        kevent_register(self.kq, changes, &[libc::EPIPE as Data])
+        kevent_register(self.kq.as_raw_fd(), changes, &[libc::EPIPE as i64])
     }
 
     pub fn reregister(&self, fd: RawFd, token: Token, interests: Interest) -> io::Result<()> {
@@ -193,9 +187,9 @@ impl Selector {
         //
         // For the explanation of ignoring `EPIPE` see `register`.
         kevent_register(
-            self.kq,
+            self.kq.as_raw_fd(),
             &mut changes,
-            &[libc::ENOENT as Data, libc::EPIPE as Data],
+            &[libc::ENOENT as i64, libc::EPIPE as i64],
         )
     }
 
@@ -211,16 +205,18 @@ impl Selector {
         // the ENOENT error when it comes up. The ENOENT error informs us that
         // the filter wasn't there in first place, but we don't really care
         // about that since our goal is to remove it.
-        kevent_register(self.kq, &mut changes, &[libc::ENOENT as Data])
-    }
-
-    #[cfg(debug_assertions)]
-    pub fn register_waker(&self) -> bool {
-        self.has_waker.swap(true, Ordering::AcqRel)
+        kevent_register(self.kq.as_raw_fd(), &mut changes, &[libc::ENOENT as i64])
     }
 
     // Used by `Waker`.
-    #[cfg(any(target_os = "freebsd", target_os = "ios", target_os = "macos"))]
+    #[cfg(any(
+        target_os = "freebsd",
+        target_os = "ios",
+        target_os = "macos",
+        target_os = "tvos",
+        target_os = "visionos",
+        target_os = "watchos"
+    ))]
     pub fn setup_waker(&self, token: Token) -> io::Result<()> {
         // First attempt to accept user space notifications.
         let mut kevent = kevent!(
@@ -230,7 +226,8 @@ impl Selector {
             token.0
         );
 
-        syscall!(kevent(self.kq, &kevent, 1, &mut kevent, 1, ptr::null())).and_then(|_| {
+        let kq = self.kq.as_raw_fd();
+        syscall!(kevent(kq, &kevent, 1, &mut kevent, 1, ptr::null())).and_then(|_| {
             if (kevent.flags & libc::EV_ERROR) != 0 && kevent.data != 0 {
                 Err(io::Error::from_raw_os_error(kevent.data as i32))
             } else {
@@ -240,7 +237,14 @@ impl Selector {
     }
 
     // Used by `Waker`.
-    #[cfg(any(target_os = "freebsd", target_os = "ios", target_os = "macos"))]
+    #[cfg(any(
+        target_os = "freebsd",
+        target_os = "ios",
+        target_os = "macos",
+        target_os = "tvos",
+        target_os = "visionos",
+        target_os = "watchos"
+    ))]
     pub fn wake(&self, token: Token) -> io::Result<()> {
         let mut kevent = kevent!(
             0,
@@ -250,7 +254,8 @@ impl Selector {
         );
         kevent.fflags = libc::NOTE_TRIGGER;
 
-        syscall!(kevent(self.kq, &kevent, 1, &mut kevent, 1, ptr::null())).and_then(|_| {
+        let kq = self.kq.as_raw_fd();
+        syscall!(kevent(kq, &kevent, 1, &mut kevent, 1, ptr::null())).and_then(|_| {
             if (kevent.flags & libc::EV_ERROR) != 0 && kevent.data != 0 {
                 Err(io::Error::from_raw_os_error(kevent.data as i32))
             } else {
@@ -264,7 +269,7 @@ impl Selector {
 fn kevent_register(
     kq: RawFd,
     changes: &mut [libc::kevent],
-    ignored_errors: &[Data],
+    ignored_errors: &[i64],
 ) -> io::Result<()> {
     syscall!(kevent(
         kq,
@@ -285,15 +290,15 @@ fn kevent_register(
             Err(err)
         }
     })
-    .and_then(|()| check_errors(&changes, ignored_errors))
+    .and_then(|()| check_errors(changes, ignored_errors))
 }
 
 /// Check all events for possible errors, it returns the first error found.
-fn check_errors(events: &[libc::kevent], ignored_errors: &[Data]) -> io::Result<()> {
+fn check_errors(events: &[libc::kevent], ignored_errors: &[i64]) -> io::Result<()> {
     for event in events {
         // We can't use references to packed structures (in checking the ignored
         // errors), so we need copy the data out before use.
-        let data = event.data;
+        let data = event.data as _;
         // Check for the error flag, the actual error will be in the `data`
         // field.
         if (event.flags & libc::EV_ERROR != 0) && data != 0 && !ignored_errors.contains(&data) {
@@ -314,15 +319,7 @@ cfg_io_source! {
 
 impl AsRawFd for Selector {
     fn as_raw_fd(&self) -> RawFd {
-        self.kq
-    }
-}
-
-impl Drop for Selector {
-    fn drop(&mut self) {
-        if let Err(err) = syscall!(close(self.kq)) {
-            error!("error closing kqueue: {}", err);
-        }
+        self.kq.as_raw_fd()
     }
 }
 
@@ -371,14 +368,28 @@ pub mod event {
 
     pub fn is_readable(event: &Event) -> bool {
         event.filter == libc::EVFILT_READ || {
-            #[cfg(any(target_os = "freebsd", target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "freebsd",
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             // Used by the `Awakener`. On platforms that use `eventfd` or a unix
             // pipe it will emit a readable event so we'll fake that here as
             // well.
             {
                 event.filter == libc::EVFILT_USER
             }
-            #[cfg(not(any(target_os = "freebsd", target_os = "ios", target_os = "macos")))]
+            #[cfg(not(any(
+                target_os = "freebsd",
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            )))]
             {
                 false
             }
@@ -415,7 +426,10 @@ pub mod event {
             target_os = "dragonfly",
             target_os = "freebsd",
             target_os = "ios",
-            target_os = "macos"
+            target_os = "macos",
+            target_os = "tvos",
+            target_os = "visionos",
+            target_os = "watchos",
         ))]
         {
             event.filter == libc::EVFILT_AIO
@@ -424,7 +438,10 @@ pub mod event {
             target_os = "dragonfly",
             target_os = "freebsd",
             target_os = "ios",
-            target_os = "macos"
+            target_os = "macos",
+            target_os = "tvos",
+            target_os = "visionos",
+            target_os = "watchos",
         )))]
         {
             false
@@ -460,7 +477,10 @@ pub mod event {
                 target_os = "freebsd",
                 target_os = "dragonfly",
                 target_os = "ios",
-                target_os = "macos"
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
             ))]
             libc::EVFILT_FS,
             #[cfg(target_os = "freebsd")]
@@ -469,7 +489,10 @@ pub mod event {
                 target_os = "freebsd",
                 target_os = "dragonfly",
                 target_os = "ios",
-                target_os = "macos"
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
             ))]
             libc::EVFILT_USER,
             #[cfg(target_os = "freebsd")]
@@ -478,9 +501,21 @@ pub mod event {
             libc::EVFILT_EMPTY,
             #[cfg(target_os = "dragonfly")]
             libc::EVFILT_EXCEPT,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::EVFILT_MACHPORT,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::EVFILT_VM,
         );
 
@@ -504,12 +539,32 @@ pub mod event {
             libc::EV_FLAG1,
             libc::EV_ERROR,
             libc::EV_EOF,
+            // Not stable across OS versions on OpenBSD.
+            #[cfg(not(target_os = "openbsd"))]
             libc::EV_SYSFLAGS,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::EV_FLAG0,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::EV_POLL,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::EV_OOBAND,
             #[cfg(target_os = "dragonfly")]
             libc::EV_NODATA,
@@ -526,49 +581,70 @@ pub mod event {
                 target_os = "dragonfly",
                 target_os = "freebsd",
                 target_os = "ios",
-                target_os = "macos"
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
             ))]
             libc::NOTE_TRIGGER,
             #[cfg(any(
                 target_os = "dragonfly",
                 target_os = "freebsd",
                 target_os = "ios",
-                target_os = "macos"
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
             ))]
             libc::NOTE_FFNOP,
             #[cfg(any(
                 target_os = "dragonfly",
                 target_os = "freebsd",
                 target_os = "ios",
-                target_os = "macos"
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
             ))]
             libc::NOTE_FFAND,
             #[cfg(any(
                 target_os = "dragonfly",
                 target_os = "freebsd",
                 target_os = "ios",
-                target_os = "macos"
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
             ))]
             libc::NOTE_FFOR,
             #[cfg(any(
                 target_os = "dragonfly",
                 target_os = "freebsd",
                 target_os = "ios",
-                target_os = "macos"
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
             ))]
             libc::NOTE_FFCOPY,
             #[cfg(any(
                 target_os = "dragonfly",
                 target_os = "freebsd",
                 target_os = "ios",
-                target_os = "macos"
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
             ))]
             libc::NOTE_FFCTRLMASK,
             #[cfg(any(
                 target_os = "dragonfly",
                 target_os = "freebsd",
                 target_os = "ios",
-                target_os = "macos"
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
             ))]
             libc::NOTE_FFLAGSMASK,
             libc::NOTE_LOWAT,
@@ -578,24 +654,54 @@ pub mod event {
             libc::NOTE_OOB,
             #[cfg(target_os = "openbsd")]
             libc::NOTE_EOF,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_EXTEND,
             libc::NOTE_ATTRIB,
             libc::NOTE_LINK,
             libc::NOTE_RENAME,
             libc::NOTE_REVOKE,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_NONE,
             #[cfg(any(target_os = "openbsd"))]
             libc::NOTE_TRUNCATE,
             libc::NOTE_EXIT,
             libc::NOTE_FORK,
             libc::NOTE_EXEC,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_SIGNAL,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_EXITSTATUS,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_EXIT_DETAIL,
             libc::NOTE_PDATAMASK,
             libc::NOTE_PCTRLMASK,
@@ -603,55 +709,147 @@ pub mod event {
                 target_os = "dragonfly",
                 target_os = "freebsd",
                 target_os = "netbsd",
-                target_os = "openbsd"
+                target_os = "openbsd",
             ))]
             libc::NOTE_TRACK,
             #[cfg(any(
                 target_os = "dragonfly",
                 target_os = "freebsd",
                 target_os = "netbsd",
-                target_os = "openbsd"
+                target_os = "openbsd",
             ))]
             libc::NOTE_TRACKERR,
             #[cfg(any(
                 target_os = "dragonfly",
                 target_os = "freebsd",
                 target_os = "netbsd",
-                target_os = "openbsd"
+                target_os = "openbsd",
             ))]
             libc::NOTE_CHILD,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_EXIT_DETAIL_MASK,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_EXIT_DECRYPTFAIL,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_EXIT_MEMORY,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_EXIT_CSERROR,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_VM_PRESSURE,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_VM_PRESSURE_TERMINATE,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_VM_PRESSURE_SUDDEN_TERMINATE,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_VM_ERROR,
-            #[cfg(any(target_os = "freebsd", target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "freebsd",
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_SECONDS,
             #[cfg(any(target_os = "freebsd"))]
             libc::NOTE_MSECONDS,
-            #[cfg(any(target_os = "freebsd", target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "freebsd",
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_USECONDS,
-            #[cfg(any(target_os = "freebsd", target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "freebsd",
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_NSECONDS,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
-            #[cfg(any(target_os = "freebsd", target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_ABSOLUTE,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_LEEWAY,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_CRITICAL,
-            #[cfg(any(target_os = "ios", target_os = "macos"))]
+            #[cfg(any(
+                target_os = "ios",
+                target_os = "macos",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos"
+            ))]
             libc::NOTE_BACKGROUND,
         );
 
